 /*
  * Khoros: $Id: vconj.c,v 1.1 1991/05/10 15:45:38 khoros Exp $
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: vconj.c,v 1.1 1991/05/10 15:45:38 khoros Exp $";
#endif

 /*
  * $Log: vconj.c,v $
 * Revision 1.1  1991/05/10  15:45:38  khoros
 * Initial revision
 *
  */ 

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1991, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as too the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1991 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: vconj.c
 >>>>
 >>>>      Program Name: vconj
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	Complex conjugate an image, pixel by pixel.
 >>>>	
 >>>>	
 >>>>            Author: Shirley Lee
 >>>>
 >>>> Date Last Updated: Sat Mar 23 23:30:32 1991
 >>>>
 >>>>          Routines: main- the main program for vconj
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vconj.h"


/****************************************************************
 *
 * Routine Name:  main program for vconj
 *
 *       Input:  
 *       -i  input image 
 *       -o  resulting output image 
 *
 *
 *       [-m] complex conjugate operation masking image
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
    struct xvimage *f1, *readimage();
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
   if (check_args()) exit(1);

   f1 = readimage(vconj->i_file);
   if (f1 == NULL) exit(1);          /* Quit if bad image */
 
   CHECKINPUT(program,f1);  /* Verify input images */
/* -main_before_lib_call_end */

/* -main_library_call */
   if (! lvconj(f1) )
   {
      (void) fprintf(stderr, "lvconj Failed\n");
      exit(1);
   }
/* -main_library_call_end */

/* -main_after_lib_call */
   writeimage(vconj->o_file,f1);
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for vconj 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "vconj :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\tComplex conjugate an image, pixel by pixel.\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i  input image  (infile)\n");
	fprintf(stderr,"\t-o  resulting output image  (outfile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-m]  complex conjugate operation masking image (infile) [null]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for vconj\n");
	fprintf(stderr, "\t[-U] Gives the usage for vconj\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called vconj.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses vconj.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for vconj
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i_pstr = 
	"\nEnter: (i) input image \n        {infile}: "; 
char	*o_pstr = 
	"\nEnter: (o) resulting output image \n        {outfile}: "; 
char	*m_pstr = 
	"\nEnter: (m) complex conjugate operation masking image \n        {infile}: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "vconj", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "vconj.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "vconj: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "vconj.ans");
		}

	     }

	}

	/*
	 * allocate the vconj structure
	 */
	vconj = (vconj_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (vconj_struct));

	/*
	 * get required arguments for vconj
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(vconj->i_flag = vget_infile(fid, prompt_flag,
			"-i", i_pstr, &(vconj->i_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vconj->i_flag))
		{
		    fprintf(stderr, "\t\ti is required, please re-enter: \n");
		    vconj->i_flag = vget_infile(fid, prompt_flag,
		    "-i", i_pstr, &(vconj->i_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(vconj->o_flag = vget_outfile(fid, prompt_flag,
			"-o", o_pstr, &(vconj->o_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vconj->o_flag))
		{
		    fprintf(stderr, "\t\to is required, please re-enter: \n");
		    vconj->o_flag = vget_outfile(fid, prompt_flag,
		    "-o", o_pstr, &(vconj->o_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-o' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for vconj
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	vconj->m_flag = vget_infile(fid, prompt_flag,
	"-m", m_pstr, &(vconj->m_file))
;


}



